;----------------------------------------------------------------------------
;
; Min2Tray
;
; _v1.0, 2005/07/11
;
; _minimize windows to tray-bar as individual icons
;  by pressing third mouse button over titlebar of window
;
; _program is a 2-in-1 thingy:
;   .started without parameters on cmdline it behaves as
;    the main program waiting for hotkey action
;   .startet with ahk_window_id on cmdline by main program
;    it will hide that particular window and generate
;    a (hopefully) meaningful tray icon
;
; _licensed under the terms of the GPLv2 or later 
;
; _created by Junyx / KTC^brain in June 2005
;
; _last modified 2005/07/11
;
; _inspired by: ActualWindowMinimizer (unfortunately payware)
;               minimize.ahk (from AHK docs, no author)
;               NiftyWindows by Enovatic-Solutions (nice tweaks)
;               ac'tivAid by Heise Zeitschriften Verlag GmbH & Co. KG
;
; _thanks to: Chris Mallett (support response in a flash!)
;             Rajat (for some wonderful scripts)
;             GSX-R600 / KTC^game (for testing)
;
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
; init
;

#Persistent
#NoTrayIcon
#SingleInstance off

Gosub, h_InitGeneral

If ( %0% < 1 )	; cmdline: no args -> behaviour: starter
{
	Gosub, h_InitStarter	; started as hotkey-launcher (starter)
	Return			; end of auto-execution for starter here!
}

;----------------------------------------------------------------------------
; Helper -- init
;

If %1% is not xdigit
	Goto, h_ExitWithParamError

h_ID = %1%				; first cmdline parameter
h_RegSubkey = Software\KTC\Min2Tray	; key in HKCU to store stuff under

;----------------------------------------------------------------------------
; Helper -- main -- hide the window
;

WinWait, ahk_id %h_ID%,, 2	; set "last used window"
If ErrorLevel <> 0  		; It timed out, so do nothing
	ExitApp, 1

WinGetClass, h_Class
If h_Class in Shell_TrayWnd,Progman
	Goto, h_ExitWithClassError

WinGetActiveTitle, h_Title
WinGet, h_Application, ProcessName

Send, !{esc}	; deactivate first, then hide window
WinHide

OnExit, h_Unhide

Gosub, h_SetMenu
Gosub, h_SetTrayIcon

Return	; end of auto execution, wait for menu action

;----------------------------------------------------------------------------
; Helper -- subs
;

h_Unhide:
 WinShow, ahk_id %h_ID%
 WinActivate, ahk_id %h_ID%
ExitApp

h_Close:
 Msgbox, 36, %lng_WindowTitle%, %lng_hClose1%%h_Title% [%h_Application%]%lng_hClose2%
 IfMsgBox, No	; do not close
	Return
 OnExit
 WinClose, ahk_id %h_ID%
ExitApp

h_ChangeIcon:
 FileSelectFile, h_appExe, 3, , %lng_ChangeIconSelector%, Icon Files (*.ico; *.dll; *.exe)
 IfExist, %h_appExe%
 {
	f_ShowTrayIcon( h_appExe, 1 )
	; write user-changed icon to registry
	; TODO -- dialog for selecting other than icon no. 1
	RegWrite, REG_SZ, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%, %h_appExe%|1
 }
Return

h_RemoveIcon:
 RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%	; does user icon exist?
 If ( Not ErrorLevel )
 {
	Msgbox, 36, %lng_WindowTitle%, %lng_RemoveIcon%
	IfMsgBox, No	; do not remove
		Return
	RegDelete, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%
	If ( Not ErrorLevel )
		Gosub, h_SetTrayIcon
 }
Return

h_SetTrayIcon:
 RegRead, tmp, HKCU, %h_RegSubkey%, %h_Application%|%h_Class%	; first look for user icon
 If ( Not ErrorLevel )
 {
	StringSplit, tmparray, tmp, |
	h_appExe     = %tmparray1%
	h_appExeIcon = %tmparray2%
 }
 Else If ( ( h_Application = "explorer.exe" ) and ( h_Class = "ExploreWClass" ) )
 {	; special treatment for explorer windows
	h_appExe     = %A_WinDir%\system32\shell32.dll
	h_appExeIcon = 5
 }
 Else
 {	; use icon from registry-entry as default, if existing
	h_appExeIcon = 1
	RegRead, h_appExe, HKLM, SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\%h_Application%
	If ( h_appExe = "" )	; otherwise do a file search
	{
		h_appExe := f_FindFile( h_Application, A_WinDir )
		If ( h_appExe = "" )	; nothing found in windir...
			h_appExe := f_FindFile( h_Application, A_ProgramFiles )
	}
 }

 f_ShowTrayIcon( h_appExe, h_appExeIcon )
Return

h_SetMenu:
 tmp := 100 - StrLen(h_Application)	; max. useful length of tray title about 100
 StringLeft, h_ShortenedTitle, h_Title, tmp

 h_MenuWin    = %h_ShortenedTitle% [%h_Application%]
 h_TrayTitle  = %h_ShortenedTitle%`n[%h_Application%]

 Menu, TRAY, Click, 1	; 1-click menu
 Menu, TRAY, NoStandard
 Menu, TRAY, Add, %h_MenuWin%, h_Unhide
 Menu, TRAY, Add
 Menu, TRAY, Add, %lng_MenuChangeIcon%, h_ChangeIcon
 Menu, TRAY, Add, %lng_MenuRemoveIcon%, h_RemoveIcon
 Menu, TRAY, Add
 Menu, TRAY, Add, %lng_MenuClose%, h_Close
 Menu, TRAY, Add, %lng_MenuUnhide%, h_Unhide
 Menu, TRAY, Default, %lng_MenuUnhide%
 Menu, TRAY, Tip, %h_TrayTitle%
Return

;----------------------------------------------------------------------------
; Starter -- subs
;

h_InitStarter:
 ; TODO
 ; check for already running instances of min2tray
 ; #SingleInstance On doesn't do the job due to 2-in-1-script!
 ;WinGet, scriptPID, PID, %A_ScriptFullPath% - AutoHotkey v	; get own PID
 ;IfWinExist, min2tray.exe	; check for min2tray already running
 ;	Goto, ExitWithRunningError

 If ( Not A_IsCompiled )
 {
	RegRead, h_ahkFullPath, HKLM, SOFTWARE\AutoHotkey, InstallDir
	IfNotExist, %h_ahkFullPath%
	{
		h_ahkFullPath := f_FindFile( "AutoHotkey.exe", A_ProgramFiles )
		If ( h_ahkFullPath = "" )
		{
			StringSplit, tmparray, A_WinDir, \	; search whole sys-drive
			h_ahkFullPath := f_FindFile( "AutoHotkey.exe", tmparray1 )
			If ( h_ahkFullPath = "" )
				Goto, ExitWithAHKError
		}
	}
 }
 
 h_StarterHotkey = MButton

 h_helperPIDs :=	; array for PIDs of started helpers
 h_hPIDcount  := 0

 DetectHiddenWindows, On
 SetWinDelay 10

 Menu, TRAY, NoStandard
 Menu, TRAY, Add, %lng_MenuAbout%, h_AboutStarter
 Menu, TRAY, Add
 Menu, TRAY, Add, %lng_MenuRestoreOnly%, h_RestoreStarter
 Menu, TRAY, Add, %lng_MenuQuitRestoreAll%, h_QuitRestoreStarter
 Menu, TRAY, Add, %lng_MenuQuitOnly%, h_QuitStarter
 Menu, TRAY, Tip, %lng_TrayTitle%

 f_ShowTrayIcon( h_StarterIcon, h_StarterIcon# )

 Hotkey, %h_StarterHotkey%, h_StartNewHelper
Return

h_StartNewHelper:
 WinWait, A,, 2		; set "last used window"
 If ErrorLevel <> 0	; It timed out, so do nothing
	Return

 CoordMode, Mouse, Relative
 SysGet, h_CaptionHeight, 4	; SM_CYCAPTION
 SysGet, h_BorderHeight, 7	; SM_CXDLGFRAME
 MouseGetPos, h_MouseX, h_MouseY

 If ( ( h_MouseY > 0 ) And ( h_MouseY < h_CaptionHeight + h_BorderHeight ) )
 {
	WinGetPos, , , h_WindowWidth
	If ( ( h_MouseX > 0 ) And ( h_MouseX < h_WindowWidth ) )
	{
		WinGet, h_WinStyle, Style

		; checks for minimizable window: (WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX)
		If ( h_WinStyle & 0xCA0000 = 0xCA0000 )
		{
			WinGet, h_ID, ID

			If ( A_IsCompiled )
				Run, %A_ScriptFullPath% %h_ID%, , UseErrorLevel, h_newPID
			Else
				Run, %h_ahkFullPath% %A_ScriptFullPath% %h_ID%, , UseErrorLevel, h_newPID

			If ( ErrorLevel = ERROR )	; helper could not be started
				Return

			h_hPIDcount++
			h_helperPIDs%h_hPIDcount% = %h_newPID%
		}
	}
 }
 Else
 {
 	MouseClick, Middle
 }
Return

h_AboutStarter:
 Msgbox, 64, %lng_WindowTitle%, %lng_About%
Return

h_RestoreStarter:	; close all helper -> unhide every window
 Loop
 {
	If ( h_hPIDcount = 0 )
		Break
	h_process := h_helperPIDs%h_hPIDcount%
	Process, Exist, %h_process%
	If ( ErrorLevel <> 0 )	; helper still exists
	{
		WinClose, ahk_pid %h_process%
	}
	h_hPIDcount--
	Sleep, 50
 }
Return

h_QuitRestoreStarter:
 Gosub, h_RestoreStarter
ExitApp

h_QuitStarter:
ExitApp

;----------------------------------------------------------------------------
; general -- subs
;

h_InitGeneral:
 StringRight, tmp, A_Language, 2
 If ( tmp = 07 )	; = Deutsch (0407, 0807, 0c07 ...)
 {
	; General
	lng_WindowTitle			= Min2Tray
	; Helper
	lng_MenuChangeIcon		= &Benutzerdefiniertes Icon...
	lng_MenuRemoveIcon		= &Entferne benuterdefiniertes Icon
	lng_MenuClose				= &Schliee Fenster
	lng_MenuUnhide				= &Zeige Fenster
	lng_hClose1					= Wollen Sie das Fenster "
	lng_hClose2					= " wirklich schlieen?
	lng_ChangeIconSelector	= Whle eine Icon-Datei aus
	lng_RemoveIcon				= Wollen Sie das benutzerdefinierte Icon wirklich entfernen?
	; Starter
	lng_MenuAbout				= &ber %lng_WindowTitle%
	lng_MenuRestoreOnly		= &Alle Fenster wiederherstellen
	lng_MenuQuitRestoreAll	= &Wiederherstellen && beenden
	lng_MenuQuitOnly			= Nur &Beenden
	lng_TrayTitle				= %lng_WindowTitle% - Haupt
	lng_About					= %lng_WindowTitle%`n`nMinimiert Fenster in den Tray-Bereich als kleine Icons.`nDazu den dritten Mausknopf ber der Titelleiste des aktiven Fensters drcken.`n`nVertrieben unter den Auflagen der GPLv2 oder neuer.`n 2005 bei Junyx / KTC^brain`n
	; Error
	lng_ExitWithParamError	= Fehler: Keine Nummer als erster Parameter bergeben!
	lng_ExitWithClassError	= Fehler: Der Desktop und das Tray-Men knnen nicht versteckt werden!
	lng_ExitWithAHKError		= Fehler: Konnte Datei "AutoHotkey.exe" nicht ausfindig machen!
	;lng_ExitWithRunningError= Fehler: Min2Tray wurde schon einmal gestartet!
 }
 Else
 {
	; General
	lng_WindowTitle			= Min2Tray
	; Helper
	lng_MenuChangeIcon		= &User defined icon...
	lng_MenuRemoveIcon		= &Remove user defined icon
	lng_MenuClose				= &Close window
	lng_MenuUnhide				= &Show window
	lng_hClose1					= Do you really wanna close window "
	lng_hClose2					= "?
	lng_ChangeIconSelector	= Select a Icon file
	lng_RemoveIcon				= Do you really wanna remove the user defined icon?
	; Starter
	lng_MenuAbout				= &About %lng_WindowTitle%
	lng_MenuRestoreOnly		= &Restore all windows
	lng_MenuQuitRestoreAll	= Quit && restore &all
	lng_MenuQuitOnly			= &Quit only
	lng_TrayTitle				= %lng_WindowTitle% - Main
	lng_About					= %lng_WindowTitle%`nMinimize windows to the Tray area as icons.`nPress third mouse button over titlebar of active window.`n`nDistributed under the terms of GPLv2 or newer.`n 2005 by Junyx / KTC^brain`n
	; Error
	lng_ExitWithParamError	= Error: First parameter must be a number!
	lng_ExitWithClassError	= Error: Desktop and Tray Menu can not be hidden!
	lng_ExitWithAHKError		= Error: Could not find file "AutoHotkey.exe"!
	;lng_ExitWithRunningError= Error: Min2Tray is already running!
 }

 ; tray icon for starter and as default
 If ( A_IsCompiled )
 {
	h_StarterIcon  = %A_ScriptFullPath%
	h_StarterIcon# = 1
 }
 Else
 {
	h_StarterIcon  = %A_WinDir%\system32\shell32.dll
	h_StarterIcon# = 25
 }
Return

;----------------------------------------------------------------------------
; general -- functions
;

f_FindFile( name, startPath )
{
 StringRight, tmp, startPath, 1
 If ( ( tmp <> "" ) and ( tmp <> "\" ) )
	startPath = %startPath%\
 Loop, %startPath%%name%, 0, 1	; start file search
 {
	appExe = %A_LoopFileLongPath%
	IfExist, %appExe%
		Break		; exit with first found app or ""
 }
 Return %appExe%
}

f_ShowTrayIcon( file, number )
{
 Global h_StarterIcon, h_StarterIcon#

 Menu, TRAY, UseErrorLevel
 Menu, TRAY, NoIcon

 IfExist, %file%
 {
	Menu, TRAY, Icon, %file%, %number%
	If ( Not ErrorLevel )	; if ErrorLevel was NOT raised -> allright
		Goto, OKout
 }

 ; if ErrorLevel or not existing file -> set default icon
 Menu, TRAY, Icon, %h_StarterIcon%, %h_StarterIcon#%

OKout:
 Menu, TRAY, Icon	; show icon
 Menu, TRAY, UseErrorLevel, Off
}


;----------------------------------------------------------------------------
; general error -- subs
;

h_ExitWithParamError:
 Msgbox, 48, %lng_WindowTitle%, %lng_ExitWithParamError%
ExitApp, 1

h_ExitWithClassError:
 Msgbox, 48, %lng_WindowTitle%, %lng_ExitWithClassError%
ExitApp, 1

ExitWithAHKError:
 Msgbox, 48, %lng_WindowTitle%, %lng_ExitWithAHKError%
ExitApp, 1

;ExitWithRunningError:
; Msgbox, 48, %lng_WindowTitle%, %lng_ExitWithRunningError%
;ExitApp, 1

